﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace FileTree
{
   public partial class TreeForm : Form
   {
      private MySorter sorter = new MySorter();

      private Bitmap picture = null;

      private void listView_ColumnClick(object sender, ColumnClickEventArgs e)
      {
          if (sorter.column_index == e.Column)
          {
              sorter.inv = ! sorter.inv;
          }
          else
          {
              sorter.column_index = e.Column;
              sorter.inv = false;
          }
          listView.Sort(); // zobrazit v novem poradi
      }

      public TreeForm()
      {
         InitializeComponent ();

         listView.ListViewItemSorter = sorter;
         listView.Sorting = SortOrder.Ascending; // nezbytne

         runButton_Click (null, null);

         /*
         foreach (MyTreeNode n in treeView.Nodes)
         {
             showStatus(n.Path);
             // if (n.Path.StartsWith ("C:"))
                 foreach (MyTreeNode m in n.Nodes)
                 {

                     showStatus(m.Path);
                     if (m.Path == "C:\\Work")
                     {
                         treeView.SelectedNode = m;
                     }
                 }
         }
          */

         // ListViewItem item = new ListViewItem();
         // item.Text = "Abc";
         // listView.Items.Add (item);
      }

      private void runButton_Click (object sender, EventArgs e)
      {
         DriveInfo [] drives = DriveInfo.GetDrives ();
         foreach ( DriveInfo d in drives )
         {
            MyTreeNode n = new MyTreeNode ();
            n.Text = d.Name;
            n.Path = d.Name;
            try
            {
               // n.ToolTipText = d.VolumeLabel;
            }
            catch ( Exception ex )
            {
               n.ToolTipText = ex.Message;
            }
            treeView.Nodes.Add (n);

            // addSubdirs (n, 2);
         }
      }

      private void addSubdirs (MyTreeNode top, int level)
      {
         if (top.ready)
         {
            if (level > 1)
               foreach (MyTreeNode n in top.Nodes)
                  addSubdirs (n, level-1);
         }
         else
         {
            top.ready = true; // nezapomenout
            try
            {
               DirectoryInfo top_dir = new DirectoryInfo (top.Path);
               DirectoryInfo [] subdirs = top_dir.GetDirectories ();
               foreach (DirectoryInfo d in subdirs)
               {
                  MyTreeNode n = new MyTreeNode ();
                  n.Text = d.Name;
                  n.ToolTipText = d.FullName;
                  n.Path = d.FullName;
                  top.Nodes.Add (n);
                  // showStatus (n.Path);

                  if ( level > 1 )
                     addSubdirs (n, level-1);
               }
            }
            catch (Exception ex)
            {
               // top.ToolTipText = ex.Message;
               // showStatus (ex.Message);
            }
         }
      }

      private void showFiles (MyTreeNode top)
      {
         listView.Items.Clear();
         try
         {
            DirectoryInfo top_dir = new DirectoryInfo (top.Path);
            FileInfo [] files = top_dir.GetFiles ();
            foreach (FileInfo f in files)
            {
               MyListNode n = new MyListNode ();
               n.FilePath = f.FullName;
               n.FileName = f.Name;
               n.FileSize = f.Length;
               n.FileTime = f.LastWriteTime;

               n.Text = n.FileName;
               n.ToolTipText = n.FilePath;

               n.SubItems.Add("" + n.FileSize);

               DateTime t = n.FileTime;
               // n.SubItems.Add(t.ToString("u"));
               n.SubItems.Add(t.ToString("dd-MM-yyyy HH:mm:ss"));

               listView.Items.Add (n);
            }
         }
         catch ( Exception ex )
         {
            // top.ToolTipText = ex.Message;
            // showStatus (ex.Message);
         }
      }

      private void showFile (MyListNode n)
      {
          if (n != null)
          {
              string ext = Path.GetExtension (n.FilePath);
              ext = ext.ToLower ();
              switch (ext)
              {
                  case ".txt":
                  case ".bat":
                  case ".log":
                  case ".pas":
                  case ".c":
                  case ".cpp":
                      textBox.LoadFile (n.FilePath, RichTextBoxStreamType.PlainText);
                      tabControl.SelectedTab = textTab;
                      break;

                  case ".rtf":
                      textBox.LoadFile (n.FilePath);
                      tabControl.SelectedTab = textTab;
                      break;

                  case ".bmp":
                  case ".jpg":
                  case ".jpeg":
                  case ".gif":
                  case ".png":
                      picture = new Bitmap(n.FilePath);
                      pictureBox_SizeChanged(null, null);
                      tabControl.SelectedTab = pictureTab;
                      break;

                  case ".htm":
                  case ".html":
                      htmlBox.Url = new Uri ("file://" + n.FilePath);
                      tabControl.SelectedTab = htmlTab;
                      break;
              }
          }
      }

      private void showStatus(string s)
      {
         statusField.Text = s;
      }

      private void treeView_BeforeSelect (object sender, TreeViewCancelEventArgs e)
      {
         MyTreeNode n = e.Node as MyTreeNode;
         addSubdirs (n, 2);
         // showStatus ("Before select " + n.Path);
      }

      private void treeView_BeforeExpand (object sender, TreeViewCancelEventArgs e)
      {
         MyTreeNode n = e.Node as MyTreeNode;
         addSubdirs (n, 2);
         // showStatus ("Before expand " + n.Path);
      }

      private void treeView_AfterSelect (object sender, TreeViewEventArgs e)
      {
         MyTreeNode n = e.Node as MyTreeNode;
         showFiles (n);
         // showStatus ("Selected " + n.Path);
      }

      private void listView_Click(object sender, EventArgs e)
      {
          if (listView.SelectedItems.Count != 0)
          {
              MyListNode n = listView.SelectedItems[0] as MyListNode;
              showFile (n);
          }
      }

      private void pictureBox_SizeChanged (object sender, EventArgs e)
      {
          if (picture != null)
          {
              int w0 = picture.Width;
              int h0 = picture.Height;
              int w = pictureBox.Width;
              int h = pictureBox.Height;
              int h1 = (int) ( (float)(h0) * w / w0 );
              if (h > h1)
                  h = h1;
              else
                  w = (int) ( (float)(w0) * h / h0 );

              pictureBox.Image = new Bitmap (picture, w, h);
          }
      }


   }
}
